/**
 * test-webid
 * Copyright (C) 2011 Thomas Bergwinkl
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bergnet.test.webid;

import java.io.File;
import java.io.FileInputStream;
import java.lang.reflect.Constructor;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Properties;

import org.bergnet.test.EarlJunitRunListener;
import org.bergnet.test.EarlTestCriterionRDFBuilder;
import org.bergnet.test.webid.publishers.FOAFPublisher;
import org.junit.runner.JUnitCore;


@SuppressWarnings("rawtypes")
public class WebIDTests {
	public static final String DEFAULT_CONFIG = "default.properties";
	public static final String OPTION_CONFIG = "config";
	public static final String OPTION_CRITERIONS = "criterions";
	public static final String OPTION_OUTPUT_FORMAT = "outputformat";
	public static final String OPTION_ENDPOINT = "endpoint";
	public static final String OPTION_BASEURL = "baseurl";
	public static final String OPTION_STAY_ALIVE = "stayalive";

	private static WebIDTests instance = null;

	private Config config = null;
	private Date startDate = new Date();
	private String outputFolder = null;
	private FOAFPublisher publisher = null;

	private WebIDTests() throws WebIDTestException {
		config = new Config(DEFAULT_CONFIG);
		config.check();
	}

	private WebIDTests(String configFileName) throws WebIDTestException {
		config = new Config(configFileName);
	}

	public static WebIDTests getInstance() throws WebIDTestException {
		if(instance == null)
			instance = new WebIDTests();

		return instance;
	}

	public void setConfig(Config config) {
		this.config = config;
	}

	public Config getConfig() {
		return config;
	}

	public Date getStartDate() {
		return startDate;
	}

	public String getOutputFolder() throws WebIDTestException {
		if(outputFolder == null) {
			try {
				SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
				outputFolder = dateFormat.format(getInstance().getStartDate());
				File outputFolderFile = new File(outputFolder);
				outputFolderFile.mkdir();
			} catch(Exception e) {
				throw new WebIDTestException("Can't create output folder", e);
			}
		}

		return outputFolder;
	}

	public FOAFPublisher getPublisher() throws WebIDTestException {
		if(publisher == null) {
			try {
				String publisherClassName = config.getPublisherClassName();

				if(publisherClassName == null)
					throw new WebIDTestException("No publisher class defined");

				Constructor publisherConstructor = Class.forName(publisherClassName).getConstructor(new Class[]{Config.class});
				publisher = (FOAFPublisher)publisherConstructor.newInstance(config);
			} catch(Exception e) {
				throw new WebIDTestException("Can't create publisher object", e);
			}
		}

		return publisher;
	}

	public static void main(String[] args) throws Exception {
		String configFileName = DEFAULT_CONFIG;
		boolean criterions = false;
		String outputFormat = "RDF/XML";
		URL endpoint = null;
		URL baseUrl = null;
		boolean stayAlive = false;

		int argIndex = 0;

		while(args.length > argIndex) {
			String arg = args[argIndex];

			if(!arg.startsWith("-"))
				continue;

			arg = arg.substring(1);

			if(OPTION_CONFIG.equals(arg)) {
				configFileName = args[++argIndex];
			} else if(OPTION_CRITERIONS.equals(arg)) {
				criterions = true;
			} else if(OPTION_OUTPUT_FORMAT.equals(arg)) {
				outputFormat = args[++argIndex];

				if(!"RDF/XML".equals(outputFormat) && !"TTL".equals(outputFormat))
					throw new WebIDTestException("Valid values for outputformat: RDF/XML, TTL");
			} else if(OPTION_ENDPOINT.equals(arg)) {
				endpoint = new URL(args[++argIndex]);
			} else if(OPTION_BASEURL.equals(arg)) {
				baseUrl = new URL(args[++argIndex]);
			} else if(OPTION_STAY_ALIVE.equals(arg)) {
				stayAlive = true;
			} else {
				throw new WebIDTestException("Unknown paramter: " + arg);
			}

			argIndex++;
		}

		instance = new WebIDTests(configFileName);

		if(endpoint != null)
			instance.getConfig().setEndpoints(new URL[]{endpoint});

		if(baseUrl != null)
			instance.getConfig().setBaseUrl(baseUrl);

		if(stayAlive)
			instance.getConfig().setStayAlive(stayAlive);

		instance.getConfig().check();

		Class[] testClasses = instance.getTestClasses();

		if(!criterions) {
			instance.runTests(testClasses, outputFormat);
		} else {
			instance.buildCriterions(testClasses, outputFormat);
		}
	}

	private void runTests(Class[] testClasses, String outputFormat) throws WebIDTestException {
		JUnitCore junitCore = new JUnitCore();

		EarlJunitRunListener earlJunitListener = new EarlJunitRunListener();
		junitCore.addListener(earlJunitListener);

		for(URL endpoint : getInstance().getConfig().getEndpoints()) {
			getInstance().getConfig().setEndpoint(endpoint);
			earlJunitListener.setSubject(endpoint.toString());

			for(Class testClass : testClasses) {
				if(instance.getConfig().isDebugEnabled())
					System.err.println("start test: " + testClass.getSimpleName());
					
				junitCore.run(testClass);
			}
		}

		if(!getInstance().getConfig().getStayAlive())
			getInstance().getPublisher().stop();

		earlJunitListener.getModel().write(System.out, outputFormat);
	}

	private void buildCriterions(Class[] testClasses, String outputFormat) throws WebIDTestException {
		EarlTestCriterionRDFBuilder.build(testClasses).write(System.out, outputFormat);
	}

	private Class[] getTestClasses() throws Exception {
		String[] testClassNames = config.getTestClassNames();

		ArrayList<Class> testClasses = new ArrayList<Class>();

		for(String testClassName : testClassNames)
			testClasses.add(Class.forName(testClassName));

		return testClasses.toArray(new Class[]{});
	}

	public static class Config {
		public static final String PROPERTY_BASE_URL = "org.bergnet.commons.test.webid.publish.baseUrl";
		public static final String PROPERTY_ENDPOINT = "org.bergnet.commons.test.webid.endpoint";
		public static final String PROPERTY_DEBUG = "org.bergnet.commons.test.webid.debug";
		public static final String PROPERTY_PUBLISHER_CLASS = "org.bergnet.commons.test.webid.publish.class";
		public static final String PROPERTY_TEST_CLASSES = "org.bergnet.commons.test.webid.testclasses";
		public static final String PROPERTY_STAY_ALIVE = "org.bergnet.test.webid.stayalive";

		private Properties properties = null;
		private URL endpoint = null;

		public Config(String fileName) throws WebIDTestException {
			try {
				properties = new Properties();
				properties.load(new FileInputStream(fileName));
			} catch(Exception e) {
				throw new WebIDTestException("Can't load properties from file");
			}
		}

		public void check() throws WebIDTestException {
			if(getBaseUrl() == null)
				throw new WebIDTestException("Base url not defined");

			if(getEndpoints() == null || getEndpoints().length == 0)
				throw new WebIDTestException("No endpoinds defined");

			if(getPublisherClassName() == null)
				throw new WebIDTestException("No publisher defined");
		}

		public Properties getProperties() {
			return properties;
		}

		public boolean isDebugEnabled() {
			String debug = properties.getProperty(PROPERTY_DEBUG);

			return ("true".equals(debug));
		}

		public void setBaseUrl(URL baseUrl) {
			properties.setProperty(PROPERTY_BASE_URL, baseUrl.toString());
		}

		public URL getBaseUrl() {
			try {
				return new URL(properties.getProperty(PROPERTY_BASE_URL));
			} catch(Exception e) {}

			return null;
		}

		public void setEndpoints(URL[] endpoints) {
			String endpointsString = null;

			for(URL endpoint : endpoints) {
				if(endpointsString == null)
					endpointsString = endpoint.toString();
				else
					endpointsString += "," + endpoint.toString();
			}

			properties.setProperty(PROPERTY_ENDPOINT, endpointsString);
		}

		public URL[] getEndpoints() {
			try {
				String[] endpointStrings = properties.getProperty(PROPERTY_ENDPOINT).split(",");
				URL[] endpoints = new URL[endpointStrings.length];

				for(int i=0; i<endpointStrings.length; i++)
					endpoints[i] = new URL(endpointStrings[i]);

				return endpoints;
			} catch(Exception e) {}

			return null;
		}

		public void setEndpoint(URL endpoint) {
			this.endpoint = endpoint;
		}

		public URL getEndpoint() {
			if(endpoint == null) {
				URL[] endpoints = getEndpoints();

				if(endpoints.length > 0)
				endpoint = endpoints[0];
			}

			return endpoint;
		}

		public String getPublisherClassName() {
			return properties.getProperty(PROPERTY_PUBLISHER_CLASS);
		}

		public String[] getTestClassNames() {
			return properties.getProperty(PROPERTY_TEST_CLASSES).split(",");
		}

		public void setStayAlive(boolean stayAlive) {
			properties.setProperty(PROPERTY_STAY_ALIVE, stayAlive ? "true" : "false");
		}

		public boolean getStayAlive() {
			return ("true".equals(properties.getProperty(PROPERTY_STAY_ALIVE)));
		}
	}
}
